package cz.muni.fi.pb162.junit;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;

/**
 * Třída reprezentuje osobu
 *
 * @author Petr Adamek
 */
public class Person {
    
    private Date birthdate;
    private String name;
    
    /**
     * Vytvoří novou instanci osoby s daným jménem a datem narození
     *
     * @param birthDate Datum narození
     * @param name jméno
     */
    public Person(Date birthDate, String name) {
	this.birthdate = birthDate;
	this.name = name;
    }
    
    /**
     * Vrátí datum narození osoby
     *
     * @return datum narození osoby
     */
    public Date getBirthdate() {
	return birthdate;
    }
    
    /**
     * Vrátí jméno osoby
     *
     * @return jméno osoby
     */
    public String getName() {
	return name;
    }
    
    /**
     * Tato metoda vrátí věk osoby v letech, přičemž platí, že každý další rok
     * je dovršen o půlnoci v den narozenin (tj. o půlnoci má osoba ještě
     * původní věk a první milisekundu po půlnoci je už o rok starší).
     *
     * Pozor! Metoda využívá nastavení Locale a časové zóny podle nastavení
     * systému.
     *
     * @param when Okamžik, ve kterém nás zajímá věk osoby
     * @return věk osoby
     */
    public int getAge(Date when) {
	Calendar birthdateCalendar = Calendar.getInstance();
	Calendar whenCalendar = Calendar.getInstance();
	birthdateCalendar.setTime(birthdate);
	birthdateCalendar.set(Calendar.MILLISECOND,1);
	birthdateCalendar.set(Calendar.SECOND,0);
	birthdateCalendar.set(Calendar.MINUTE,0);
	birthdateCalendar.set(Calendar.HOUR_OF_DAY,0);
	whenCalendar.setTime(when);
	
	int result = whenCalendar.get(Calendar.YEAR) -
		birthdateCalendar.get(Calendar.YEAR);
	whenCalendar.add(Calendar.YEAR,-result);
	if (whenCalendar.before(birthdateCalendar)) {
	    result--;
	}
	return result;
    }
    
    /**
     * Metoda vrátí true pokud je tento objekt porovnáván se stejnou osobou, 
     * což v tomto případě znamená, že má stejné jméno a stejné datum narození
     *
     * @param obj objekt k porovnání
     * @return true pokud se jedná o stejnou osobu
     */
    public boolean equals(Object obj) {
	return (obj instanceof Person) &&
		((Person) obj).getBirthdate().equals(getBirthdate()) &&
		((Person) obj).getName().equals(getName());
    }
    
    /**
     * Vrátí hashovací kód
     *
     * @return hashovcací kód
     */
    public int hashCode() {
	int result = 17;
	result = 37*result + getBirthdate().hashCode();
	result = 37*result + getName().hashCode();
	return result;
    }
    
}
