package cz.muni.fi.pb162.junit;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;

public class TestClassLoader extends ClassLoader {
    
    private static final byte[] classData1 = {
	-54, -2, -70, -66,      // Magic header 0xCA 0xFE 0xBA 0xBE
	0x00, 0x00, 0x00, 0x31, // Major and minor version number
	// Constant pool
	0x00, 0x0A,             // Size of constant pool
	0x0A, 0x00, 0x03, 0x00, 0x07,  // [1] CONSTANT_Methodref
	0x07, 0x00, 0x08,              // [2] CONSTANT_Class
	0x07, 0x00, 0x09,              // [3] CONSTANT_Class
	0x01, 0x00, 0x06,              // [4] CONSTANT_Utf8 "<init>"
	0x3C, 0x69, 0x6E, 0x69, 0x74, 0x3E,
	0x01, 0x00, 0x25,              // [5] CONSTANT_Utf8 "(Ljava/util/Date;Ljava/lang/String;)V"
	0x28, 0x4C, 0x6A, 0x61, 0x76, 0x61, 0x2F, 0x75,
	0x74, 0x69, 0x6C, 0x2F, 0x44, 0x61, 0x74, 0x65,
	0x3B, 0x4C, 0x6A, 0x61, 0x76, 0x61, 0x2F, 0x6C,
	0x61, 0x6E, 0x67, 0x2F, 0x53, 0x74, 0x72, 0x69,
	0x6E, 0x67, 0x3B, 0x29, 0x56,
	0x01, 0x00, 0x04,               // [6] CONSTANT_Utf8 "Code"
	0x43, 0x6F, 0x64, 0x65,
	0x0C, 0x00, 0x04, 0x00, 0x05,   // [7] CONSTANT_NameAndType "<init>" (Constructor)
	0x01, 0x00, 0x1D,               // [8] CONSTANT_Utf8 "cz/muni/fi/pb162/junit/Person"
	0x63, 0x7A, 0x2F, 0x6D, 0x75, 0x6E, 0x69, 0x2F,
	0x66, 0x69, 0x2F, 0x70, 0x62, 0x31, 0x36, 0x32,
	0x2F, 0x6A, 0x75, 0x6E, 0x69, 0x74, 0x2F, 0x50,
	0x65, 0x72, 0x73, 0x6F, 0x6E,
    };
    /*
	0x01, 0x??, 0x??,               // [9] CONSTANT_Utf8 "someSuperclass"
	0x ...
     */
    private static final byte[] classData2 = {
	
	0x00, 0x21, // access flags
	0x00, 0x02, // this class
	0x00, 0x03, // super class
	0x00, 0x00, // interfaces count
	0x00, 0x00, // fields count
	// Methods
	0x00, 0x01, // methods count
	// Method <init>
	0x00, 0x01, // access flags
	0x00, 0x04, // name index
	0x00, 0x05, // descriptor index
	0x00, 0x01, // attributes count
	0x00, 0x06, 0x00, 0x00, 0x00, 0x13, // attribute Code
	0x00, 0x03, 0x00, 0x03, 0x00, 0x00, 0x00, 0x07,
	0x2A, 0x2B, 0x2C, -73 /*0xB7*/, 0x00, 0x01, -79 /*0xB1*/, 0x00,
	0x00, 0x00, 0x00,
	// Attributes
	0x00, 0x00 // attributes_count
    };
    
    private static final String className = "cz.muni.fi.pb162.junit.Person";
    private Class parentClass;
    
    /** Creates a new instance of TestClassLoader */
    public TestClassLoader(Class parentClass) {
	super(parentClass.getClassLoader());
	this.parentClass = parentClass;
    }
    
    protected synchronized Class<?> loadClass(String name, boolean resolve)
    throws ClassNotFoundException {
	Class clazz = findLoadedClass(name);
	if (clazz == null) {
	    if (name.startsWith("cz.muni.fi.pb162.junit")) {
		clazz = findClass(name);
	    } else {
		clazz = getParent().loadClass(name);
	    }
	}
	if (resolve) {
	    resolveClass(clazz);
	}
	return clazz;
    }
    
    public Class findClass(String name) throws ClassNotFoundException {
	byte[] b;
	if (name.equals(className)) {
	    b = loadClassData();
	} else {
	    b = loadClassDataFromParentClassLoader(name);
	}
        return defineClass(name, b, 0, b.length);
    }
    
    private byte[] loadClassData() {
	try {
            ByteArrayOutputStream bytes = new ByteArrayOutputStream();
	    DataOutputStream data = new DataOutputStream(bytes);
	    data.write(classData1);
	    data.writeByte(0x01);
	    data.writeUTF(parentClass.getName().replace('.','/'));
	    data.write(classData2);
	    data.flush();
            return bytes.toByteArray();
	} catch (IOException ex) {
	    // this should never happen
	    throw new RuntimeException(ex);
	}
    }
    
    private byte[] loadClassDataFromParentClassLoader(String name)
    throws ClassNotFoundException {
	InputStream is = parentClass.getClassLoader().getResourceAsStream(
		name.replace('.','/') + ".class");
	if (is != null) {
	    ByteArrayOutputStream result = new ByteArrayOutputStream();
	    byte[] buffer = new byte[1];
	    int count;
	    try {
		while ((count = is.read(buffer)) >= 0) {
		    result.write(buffer,0,count);
		}
	    } catch (IOException ex) {
		throw new ClassNotFoundException(name,ex);
	    }
	    return result.toByteArray();
	} else {
	    throw new ClassNotFoundException(name);
	}
    }
}
